import os
import json
from yv_xtract.terminal_utils import print_info, print_processing


class YVXConfig:
    def __init__(self) -> None:
        self.CONFIG_FILE = "yvx_config.json"

    def check_if_config_exists(self) -> bool:
        return os.path.exists(self.CONFIG_FILE)

    def create_config_file(self) -> None:
        print_info("Set a default path to store the downloaded files:")
        path = input()
        print_processing("Creating config file...")
        with open(self.CONFIG_FILE, "w") as f:
            path_to_assign = self.check_path(path)
            json.dump({"default_path": path_to_assign}, f)
        cwd = os.getcwd()
        print_info(f"'{self.CONFIG_FILE}' created successfully at '{cwd}'.")

    def get_default_path(self) -> str:
        with open(self.CONFIG_FILE, "r") as f:
            return json.load(f)["default_path"]

    def update_default_path(self, path: str) -> None:
        print_processing(f"Updating default path to '{path}'...")
        with open(self.CONFIG_FILE, "w") as f:
            json.dump({"default_path": path}, f)
        print_info("Default path updated successfully.")

    def check_path(self, path: str) -> str | None:
        if path is None:
            return None
        path = path.strip()
        if path[0] == '"' and path[-1] == '"':
            path = path[1:-1]
        if path[-1] != "/":
            path += "/"
        return path



from yv_xtract.terminal_utils import (
    parse_arguments,
    print_error,
    print_info,
)
from yv_xtract.yvx_config import YVXConfig
from yv_xtract.video_downloader import VideoDownloader


def main() -> None:
    args = parse_arguments()
    config = YVXConfig()

    url: str = args.url
    given_path: str | None = config.check_path(args.path)
    default_path: str | None = config.check_path(args.set_path)
    mp3: bool = args.mp3
    gif: bool = args.gif
    start_time: str | None = args.start_time
    end_time: str | None = args.end_time

    path_to_save: str = ""

    if mp3 and gif:
        print_error("Cannot convert to MP3 and GIF at the same time.")
        return

    if not config.check_if_config_exists():
        config.create_config_file()

    if default_path is not None:
        config.update_default_path(default_path)

    if given_path is None:
        path_to_save = config.get_default_path()
        print_info(f"Using default path '{path_to_save}' to store the downloaded file.")
    else:
        path_to_save = given_path
        print_info(f"Using '{given_path}' to store the downloaded file.")

    try:
        VideoDownloader(
            video_url=url,
            output_path=path_to_save,
            start_time=start_time,
            end_time=end_time,
            isMP3=mp3,
            isGIF=gif,
        ).download()
    except Exception as err:
        print_error(err)
        return


if __name__ == "__main__":
    main()



from argparse import Namespace, ArgumentParser
from termcolor import colored


def parse_arguments() -> Namespace:
    parser = ArgumentParser(
        description="Download videos from YouTube in MP4, MP3 or GIF format."
    )
    parser.add_argument("url", help="URL of the video to download.")
    parser.add_argument(
        "-mp3", action="store_true", help="Convert the video to MP3 format."
    )
    parser.add_argument(
        "-gif",
        action="store_true",
        help="Convert the video to GIF format.",
    )
    parser.add_argument(
        "--start-time",
        help="Time from where the video will start (Format: HH:MM:SS [e.g., 00:02:30]).",
    )
    parser.add_argument(
        "--end-time",
        help="Time where the video will end (Format: HH:MM:SS [e.g., 00:05:00]).",
    )
    parser.add_argument(
        "--path",
        help="Path to store the downloaded file.",
    )
    parser.add_argument(
        "--set-path",
        help="Set the default path to store the downloaded files.",
    )
    return parser.parse_args()


def print_error(message):
    print(colored(f"[ERROR]: {message}", "red"))


def print_success(message):
    print(colored(f"[SUCCESS]: {message}", "green"))


def print_processing(message):
    print(colored(f"[PROCESSING]: {message}", "yellow"))


def print_info(message):
    print(colored(f"[INFO]: {message}", "blue"))



from pytube import YouTube
from moviepy.video.io.ffmpeg_tools import ffmpeg_extract_audio, ffmpeg_extract_subclip
from yv_xtract.terminal_utils import print_success, print_processing
from moviepy.editor import VideoFileClip
import os


class VideoDownloader:
    def __init__(
        self,
        video_url: str,
        output_path: str,
        start_time: str = None,
        end_time: str = None,
        isMP3: bool = False,
        isGIF: bool = False,
    ) -> None:
        self.__url = video_url
        self.__yt = YouTube(self.__url)
        self.__current_filename = self.__yt.title
        self.__output_path = output_path
        self.__isMP3 = isMP3
        self.__start_time = start_time
        self.__end_time = end_time
        self.__isGIF = isGIF

    def __clean_up_video(self) -> None:
        os.remove(f"{self.__output_path}{self.__current_filename}.mp4")

    def __download_video(self) -> None:
        stream = self.__yt.streams.get_highest_resolution()
        if self.__isMP3:
            stream = self.__yt.streams.get_audio_only()
        stream.download(output_path=self.__output_path)

    def __time_to_seconds(self, time: str) -> int:
        if time is None:
            return 0
        h, m, s = map(int, time.split(":"))
        return h * 3600 + m * 60 + s

    def __check_time_in_bounds(self) -> None:
        video_duration = self.__yt.length
        end_time_seconds = self.__time_to_seconds(self.__end_time)

        if end_time_seconds > video_duration:
            self.__clean_up_video()
            raise Exception("End time is greater than video duration.")

    def __cut_video(self) -> None:
        if self.__start_time is None and self.__end_time is None:
            return

        video_duration = self.__yt.length

        if self.__start_time:
            start_time_parts = list(map(int, self.__start_time.split(":")))
            start_seconds = (
                start_time_parts[0] * 3600
                + start_time_parts[1] * 60
                + start_time_parts[2]
            )
        else:
            start_seconds = 0

        if self.__end_time:
            end_time_parts = list(map(int, self.__end_time.split(":")))
            end_seconds = (
                end_time_parts[0] * 3600 + end_time_parts[1] * 60 + end_time_parts[2]
            )
        else:
            end_seconds = video_duration

        self.__check_time_in_bounds()

        ffmpeg_extract_subclip(
            f"{self.__output_path}{self.__yt.title}.mp4",
            start_seconds,
            end_seconds,
            targetname=f"{self.__output_path}{self.__yt.title}(clip).mp4",
        )
        self.__clean_up_video()
        self.__current_filename = f"{self.__yt.title}(clip)"

    def __convert_to_mp3(self) -> None:
        ffmpeg_extract_audio(
            f"{self.__output_path}{self.__current_filename}.mp4",
            f"{self.__output_path}{self.__current_filename}.mp3",
        )

    def __convert_to_gif(self) -> None:
        with VideoFileClip(
            f"{self.__output_path}{self.__current_filename}.mp4"
        ) as clip:
            clip.write_gif(f"{self.__output_path}{self.__current_filename}.gif")

    def download(self) -> None:
        print_processing("Downloading video...")
        self.__download_video()
        self.__cut_video()
        if self.__isGIF:
            print_processing("Converting to GIF...")
            self.__convert_to_gif()
            self.__clean_up_video()
        if self.__isMP3:
            print_processing("Converting to MP3...")
            self.__convert_to_mp3()
            self.__clean_up_video()
        print_success("Done.")
